/* modified for 3ax DOSY experiments by KP
Developed By NMR Group
School of Chemistry
University of Manchester
United Kingdom
2015

_01	3ax option added
_02	makedosyparams updated for 3ax
_03	gradaxis='xy' was added
_04	gradsign implemented
_05	kp_corr implemented to use gradaxis[1] as a correction factor on g-recovery with gradsign='+-'/'-+'
_06	change the implementation to have reasonably identical gradient strength when walking around the XY plane
	if gradaxis='xyc' than it will apply gzlvl1*kp_corr1 on gradaxis[0] and gzlvl1*kp_corr2 on gradaxis[1]
	otherwise just apply gzlvl1 on gradaxis[0] and gradaxis[1]
_07	kpph
*/




#ifndef LINT
static char SCCSid[] = "@(#)Doneshot.c 18.2 12/23/04 Copyright (c) 1991-2004 Agilent Technologies All Rights Reserved";
#endif
/* 
 * Agilent Technologies All Rights Reserved.
 * This software contains proprietary and confidential
 * information of Agilent Technologies and its contributors.
 * Use, disclosure and reproduction is prohibited without
 * prior consent.
 */
/**********************************************************
	Doneshot - Oneshot DOSY sequence based on modifications to the
	Bipolar pulse stimulated echo (BPPSTE) sequence

Ref: M. D. Pelta, G. A. Morris, M. J. Tschedroff and S. J. Hammond:
	MRC 40, 147-152 (2002)
  For eliminating radiation damping: 
	M. A. Conell, A. L. Davis, A. M. Kenwright and G. A. Morris:
	Anal. Bioanal. Chem. 378, 1568-1573, (2004).

Parameters:
        delflag   - 'y' runs the Doneshot sequence
                    'n' runs the normal s2pul sequence
	del       -  the actual diffusion delay
	gt1       - total diffusion-encoding pulse width
        gzlvl1    - diffusion-encoding pulse strength
        gstab     - gradient stabilization delay (~0.0002-0.0003 sec)
        gt3       - spoiling gradient duration (in sec)
        gzlvl3    - spoiling gradient strength (destroys transverse 
			magnetization during the diffusion delay)
        gzlvl_max - maximum accepted gradient strength
                       32767 with PerformaII, 2047 with PerformaI
        kappa     - unbalancing factor between bipolar pulses as a
                       proportion of gradient strength (~0.2)
	startflip - flip angle of the firts pulse to eliminate
			radiation damping for very concentrated samples
        alt_grd   - flag to invert gradient sign on alternating scans
                        (default = 'n')
       lkgate_flg - flag to gate the lock sampling off during    
                              gradient pulses
        sspul     - flag for a GRD-90-GRD homospoil block
        gzlvlhs   - gradient level for sspul
        hsgt      - gradient duration for sspul
        satmode   - flag for optional solvent presaturation
                    'yn' - does presat during satdly
                    'yy' - does presat during satdly and the diffusion delay
        satdly    - presaturation delay before the sequence (part of d1)
        satpwr    - saturation power level
        satfrq    - saturation frequency
        wet       - flag for optional wet solvent suppression
        ncomp     - determines the number of components to be used in fitting
                        the signal decay in DOSY when dosyproc='discrete'.
        nugflag   - 'n' uses simple mono- or multi-exponential fitting to
                          estimate diffusion coefficients
                    'y' uses a modified Stejskal-Tanner equation in which the
                          exponent is replaced by a power series
     nugcal_[1-5] - a 5 membered parameter array summarising the results of a
                        calibration of non-uniform field gradients. Used if
                        nugcal is set to 'y'
                        requires a preliminary NUG-calibration by the 
                        Doneshot_nugmap sequence
        dosyproc  - 'discrete' - invokes monoexponential fitting with dosyfit if 
                        ncomp=1, and multiexponential fitting with the external
                        programme SPLMOD if ncomp>1.
                    'continuous' invokes processing with the external programme
                        CONTIN and gives a continuous distribution
                        in the diffusion domain.
     dosypypoints - 'n' divides the spectrum into individual peaks, creating one 
                        cross-peak for each individual peak found in the 1D spectrum
                    'y' performs a diffusion fit for every point in the displayed
                        region of the spectrum that lies above the threshold th
           probe_ - stores the probe name used for the dosy experiment

The parameters for the heating gradients (gt4, gzlvl4) are calculated
in the sequence. They cannot be set directly.
tau defined as time between the mid-points of the bipolar diffusion encoding
gradient pulses

**********************************************************/

#include <standard.h>
#include <chempack.h>
			//  1-2---4-------8---------------16--------------------------------32---------------------------------------------------------------64
static int	 ph1[256] = {0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,  
			     0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3, 
			     0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,  
			     0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3, 0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3,0,2,0,2,1,3,1,3},	//v1	- 1st 90 
		 ph2[256] = {0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,  
			     0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 
			     2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  
			     2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2},	//v2	- 1st 180 
		 ph3[256] = {0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  
			     0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
			     0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  
			     0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2},	//v3	- 2nd 90
		 ph4[256] = {0,0,2,2,0,0,2,2, 1,1,3,3,1,1,3,3, 2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1, 0,0,2,2,0,0,2,2,1,1,3,3,1,1,3,3,2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1,  
			     0,0,2,2,0,0,2,2, 1,1,3,3,1,1,3,3, 2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1, 0,0,2,2,0,0,2,2,1,1,3,3,1,1,3,3,2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1,
			     0,0,2,2,0,0,2,2, 1,1,3,3,1,1,3,3, 2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1, 0,0,2,2,0,0,2,2,1,1,3,3,1,1,3,3,2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1,  
			     0,0,2,2,0,0,2,2, 1,1,3,3,1,1,3,3, 2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1, 0,0,2,2,0,0,2,2,1,1,3,3,1,1,3,3,2,2,0,0,2,2,0,0,3,3,1,1,3,3,1,1},	//v4	- 3rd 90
		 ph5[256] = {0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  
			     1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1, 3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
			     0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,  
			     1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1, 3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3},	//v5	- 2nd 180
		 ph6[256] = {1,1,3,3,0,0,2,2, 0,0,2,2,3,3,1,1, 3,3,1,1,2,2,0,0,2,2,0,0,1,1,3,3, 3,3,1,1,2,2,0,0,2,2,0,0,1,1,3,3,1,1,3,3,0,0,2,2,0,0,2,2,3,3,1,1,  
			     3,3,1,1,2,2,0,0, 2,2,0,0,1,1,3,3, 1,1,3,3,0,0,2,2,0,0,2,2,3,3,1,1, 1,1,3,3,0,0,2,2,0,0,2,2,3,3,1,1,3,3,1,1,2,2,0,0,2,2,0,0,1,1,3,3,
			     1,1,3,3,0,0,2,2, 0,0,2,2,3,3,1,1, 3,3,1,1,2,2,0,0,2,2,0,0,1,1,3,3, 3,3,1,1,2,2,0,0,2,2,0,0,1,1,3,3,1,1,3,3,0,0,2,2,0,0,2,2,3,3,1,1,  
			     3,3,1,1,2,2,0,0, 2,2,0,0,1,1,3,3, 1,1,3,3,0,0,2,2,0,0,2,2,3,3,1,1, 1,1,3,3,0,0,2,2,0,0,2,2,3,3,1,1,3,3,1,1,2,2,0,0,2,2,0,0,1,1,3,3},	//v6	- final beta
	//v7 will be used for alternating gradients
	//v10 will be used as a ct counter
		ph10[256] = {0,2,2,0,1,3,3,1, 3,1,1,3,0,2,2,0, 2,0,0,2,3,1,1,3,1,3,3,1,2,0,0,2, 2,0,0,2,3,1,1,3,1,3,3,1,2,0,0,2,0,2,2,0,1,3,3,1,3,1,1,3,0,2,2,0,  
			     2,0,0,2,3,1,1,3, 1,3,3,1,2,0,0,2, 0,2,2,0,1,3,3,1,3,1,1,3,0,2,2,0, 0,2,2,0,1,3,3,1,3,1,1,3,0,2,2,0,2,0,0,2,3,1,1,3,1,3,3,1,2,0,0,2,
			     0,2,2,0,1,3,3,1, 3,1,1,3,0,2,2,0, 2,0,0,2,3,1,1,3,1,3,3,1,2,0,0,2, 2,0,0,2,3,1,1,3,1,3,3,1,2,0,0,2,0,2,2,0,1,3,3,1,3,1,1,3,0,2,2,0,  
			     2,0,0,2,3,1,1,3, 1,3,3,1,2,0,0,2, 0,2,2,0,1,3,3,1,3,1,1,3,0,2,2,0, 0,2,2,0,1,3,3,1,3,1,1,3,0,2,2,0,2,0,0,2,3,1,1,3,1,3,3,1,2,0,0,2};	//oph


pulsesequence()
{
double	delcor,initdel,gzlvl4,gt4,Dtau,Ddelta,dosytimecubed,dosyfrq,
        kappa     = getval("kappa"),
	kp_beta	  = getval("kp_beta"),
	kp_gsign0, kp_gsign1, 
	kp_corr0   = 0.0, //getval("kp_corr0"),
	kp_corr1   = 0.0, //getval("kp_corr1"),
	kp_angle = getval("kp_angle"),
	gzlvl1    = getval("gzlvl1"),
	//gzlvl2    = getval("gzlvl2"),
	gzlvl3    = getval("gzlvl3"),
        gzlvl_max = getval("gzlvl_max"),
	gt1       = getval("gt1"),
	gt3       = getval("gt3"),
	del       = getval("del"),
        gstab     = getval("gstab"),
	gstab0    = getval("gstab0"),
        satpwr    = getval("satpwr"),
        satdly    = getval("satdly"),
        satfrq    = getval("satfrq"),
        gzlvlhs   = getval("gzlvlhs"),
        hsgt     = getval("hsgt"),
        startflip = getval("startflip");

int 	kpph = (int)(getval("kpph")); 	//number of steps from the phase cycle to use

char delflag[MAXSTR], sspul[MAXSTR],
	satmode[MAXSTR],alt_grd[MAXSTR],lkgate_flg[MAXSTR],oneshot45_flg[MAXSTR],
	gradaxis[MAXSTR],   	// 'x'/'y'/'z'/'xy' 
				
	gradsign[MAXSTR];	//changes the sign of diffusion gradient

   gt4 = 2.0*gt1;
   getstr("delflag",delflag);
   getstr("alt_grd",alt_grd);
   getstr("oneshot45_flg",oneshot45_flg);
   getstr("satmode",satmode);
   getstr("lkgate_flg",lkgate_flg);
   getstr("sspul",sspul);
   getstr("gradaxis",gradaxis);
   getstr("gradsign",gradsign);

	if (kpph>256)
	{
	abort_message("kpph is too long...\n");
	}


/* Decrement gzlvl4 as gzlvl1 is incremented, to ensure constant 
   energy dissipation in the gradient coil 
   Current through the gradient coil is proportional to gzlvl */

   gzlvl4 = sqrt(2.0*gt1*(1+3.0*kappa*kappa)/gt4) * 
            sqrt(gzlvl_max*gzlvl_max/((1+kappa)*(1+kappa))-gzlvl1*gzlvl1);

/* In pulse sequence, del>4.0*pw+3*rof1+2.0*gt1+5.0*gstab+gt3 */

   if ((del-(4*pw+3.0*rof1+2.0*gt1+5.0*gstab+gt3)) < 0.0)
   {  del=(4*pw+3.0*rof1+2.0*gt1+5.0*gstab+gt3);
      text_message("Warning: del too short; reset to minimum value");
   }

   if ((d1 - (gt3+gstab) -2.0*(gt4/2.0+gstab)) < 0.0)
   {  d1 = (gt3+gstab) -2.0*(gt4/2.0+gstab);
      text_message("Warning: d1 too short;  reset to minimum value");
   }

   if ((gzlvl1*(1+kappa)) > gzlvl_max)
   {  abort_message("Max. grad. amplitude exceeded: reduce either gzlvl1 or kappa");
   }

   if (gradsign[0]=='-')   { kp_gsign0 = -1.0;  }
   else   { kp_gsign0 = 1.0;  }
   if (gradsign[1]=='-')   { kp_gsign1 = -1.0;  }
   else   { kp_gsign1 = 1.0;  }

   if (gradaxis[2]=='c') 
   {

  kp_corr0=cos(kp_angle/180*PI);
  kp_corr1=sin(kp_angle/180*PI);

	if 
//	 ( (sqrt((kp_corr0*kp_corr0) + (kp_corr1*kp_corr1)) ) != 1.000 )
	( ( (sqrt((kp_corr0*kp_corr0) + (kp_corr1*kp_corr1)) ) > 1.001 ) || ( ( sqrt((kp_corr0*kp_corr0) + (kp_corr1*kp_corr1)) ) < 0.999 )   )
	{
	abort_message("Sum of %f is not equal to 1.0", (sqrt((kp_corr0*kp_corr0) + (kp_corr1*kp_corr1)) ) );
	}


   kp_gsign0 = kp_gsign0 * kp_corr0;
   kp_gsign1 = kp_gsign1 * kp_corr1;
   }


   if (ni > 1.0)
   {  
   abort_message("This is a 2D, not a 3D dosy sequence: please set ni to 0 or 1");
   }

   if (delflag[0]=='y')
     { initdel=(gt3+gstab) -2.0*(gt4/2.0+gstab); }
   else
     { initdel=0; }

   Ddelta=gt1;
   Dtau=2.0*pw+gstab+rof1;
   dosyfrq = sfrq;
   dosytimecubed = Ddelta*Ddelta*(del+(Ddelta/6.0) *
                   (kappa*kappa-2.0) +
                   (Dtau/2.0)*(kappa*kappa-1.0));
   putCmd("kp_makedosyparams(%e,%e)\n",dosytimecubed,dosyfrq);

/* phase cycling calculation */

if (kpph>0)
{
//setup phase cycle up to kpph
  	assign(ct,v10);

	settable(t1,kpph,ph1);
	settable(t2,kpph,ph2);
	settable(t3,kpph,ph3);
	settable(t4,kpph,ph4);
	settable(t5,kpph,ph5);
	settable(t6,kpph,ph6);
	settable(t10,kpph,ph10);

  	getelem(t1, v10, v1);		
  	getelem(t2, v10, v2);
  	getelem(t3, v10, v3);
  	getelem(t4, v10, v4);
  	getelem(t5, v10, v5);
  	getelem(t6, v10, v6);
	getelem(t10, v10, oph);
}
else
{
//use orig. phase calculations


   if(delflag[0]=='y')
   {
	 mod2(ct,v1); dbl(v1,v1);  hlv(ct,v2);
	 mod2(v2,v3); dbl(v3,v3);  hlv(v2,v2);
	 mod2(v2,v4); add(v1,v4,v1);			/*    v1      */
	 hlv(v2,v2);  add(v2,v3,v4);			/*    v4      */
	 hlv(v2,v2);  mod2(v2,v3); dbl(v3,v5);
	 hlv(v2,v2);  mod2(v2,v3); dbl(v3,v3);		/*    v3      */
	 hlv(v2,v2);  mod2(v2,v6); add(v5,v6,v5);	/*    v5      */
	 hlv(v2,v2);  mod2(v2,v2); dbl(v2,v2);		/*    v2      */
	 assign(v1,oph);  dbl(v2,v6);      sub(oph,v6,oph);
	 add(v3,oph,oph); sub(oph,v4,oph); dbl(v5,v6);
	 add(v6,oph,oph); mod4(oph,oph);                /* receiver phase */
	 mod2(ct,v6); dbl(v6,v6); /* 02 */
	 hlv(ct,v8); hlv(v8,v8); dbl(v8,v8); mod4(v8,v8); /* 00002222 */
	 add(oph,v8,v8);
	 add(v8,v6,v6);
	 add(v6,one,v6); /*Optional 45-degree pulse before acquisition- used when oneshot45_flg='y'*/

   }
}

//alternate gradient sign with every scan
      mod2(ct,v7);     




   status(A);
   if ((lkgate_flg[0] == 'y') || (lkgate_flg[0] == 'k'))  lk_hold(); // turn off lock sampling
   delay(gstab0);

   if(delflag[0]=='y')
	{

// 1st dummy heating pulse 
	rgradient(gradaxis[0],-1.0*gzlvl4*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl4*kp_gsign1); } 
	delay(gt4/2.0); 
	rgradient(gradaxis[0],0.0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }	
	delay(gstab);

// 2nd dummy heating pulse 
	rgradient(gradaxis[0],gzlvl4*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl4*kp_gsign1); } 
	delay(gt4/2.0); 
	rgradient(gradaxis[0],0.0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }	
	delay(gstab);
   	}

     if (sspul[0]=='y')
       {
        rgradient(gradaxis[0],gzlvlhs*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvlhs*kp_gsign1); } 
	delay(hsgt); 
	rgradient(gradaxis[0],0.0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }

	rgpulse(pw,zero,rof1,rof1);

	rgradient(gradaxis[0],gzlvlhs*kp_gsign0);
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvlhs*kp_gsign1); } 
	delay(hsgt); 
	rgradient(gradaxis[0],0.0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
       }

   delay(gstab0); 
   if ((lkgate_flg[0] == 'y') || (lkgate_flg[0] == 'k'))  lk_sample(); // turn on lock sampling

   if (satmode[0] == 'y')
      {
       if (d1 - satdly > 0)
         delay(d1 - satdly);
       else
       delay(0.02);
       obspower(satpwr);
        if (satfrq != tof)
         obsoffset(satfrq);
        rgpulse(satdly,zero,rof1,rof1);
        if (satfrq != tof)
         obsoffset(tof);
       obspower(tpwr);
       delay(1.0e-5);
      }
     else
     {  delay(d1-initdel); }

 if (getflag("wet")) wet4(zero,one);

 if (delflag[0]=='y')
    {
   if ((lkgate_flg[0] == 'y') || (lkgate_flg[0] == 'k'))  lk_hold(); // turn off lock sampling

// Spoiler gradient balancing pulse
        if (alt_grd[0] == 'y')
        {
	ifzero(v7); 
		{ 
		rgradient(gradaxis[0],-1.0*gzlvl3*kp_gsign0); 	
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl3*kp_gsign1); }  
		delay(gt3); 
		rgradient(gradaxis[0],0.0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); } 
		} 
 	elsenz(v7);
		{
		rgradient(gradaxis[0],1.0*gzlvl3*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],1.0*gzlvl3*kp_gsign1); }  
		delay(gt3); 
		rgradient(gradaxis[0],0.0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
        endif(v7);
        }
	else 
	{
	rgradient(gradaxis[0],-1.0*gzlvl3*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl3*kp_gsign1); }  
	delay(gt3); 
	rgradient(gradaxis[0],0.0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}
        delay(gstab);
    }

   status(B); 
   if (delflag[0]=='y')
   {

// first 90, v1 
   	 rgpulse(startflip*pw/90.0, v1, rof1, 0.0);		

//1st main gradient pulse
	if (alt_grd[0] == 'y')
	{
	ifzero(v7); 
		{
		rgradient(gradaxis[0],gzlvl1*(1.0-kappa)*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl1*(1.0-kappa)*kp_gsign1); }  
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); } 
		}
         elsenz(v7); 
		{
		rgradient(gradaxis[0],-1.0*gzlvl1*(1.0-kappa)*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl1*(1.0-kappa)*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
         endif(v7);
        }
	else 
	{
	rgradient(gradaxis[0],gzlvl1*(1.0-kappa)*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl1*(1.0-kappa)*kp_gsign1); } 
	delay(gt1/2.0); 
	rgradient(gradaxis[0],0.0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}
	delay(gstab);

// first 180, v2 
	 rgpulse(pw*2.0, v2, rof1, 0.0);	

//2nd main grad. pulse
        if (alt_grd[0] == 'y')
        {
	ifzero(v7); 
		{
		rgradient(gradaxis[0],-1.0*gzlvl1*(1.0+kappa)*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl1*(1.0+kappa)*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); } 
		}
         elsenz(v7); 
		{
		rgradient(gradaxis[0],1.0*gzlvl1*(1.0+kappa)*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],1.0*gzlvl1*(1.0+kappa)*kp_gsign1); } 
		delay(gt1/2.0);	
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
         endif(v7);
        }
	else 
	{
	rgradient(gradaxis[0],-1.0*gzlvl1*(1.0+kappa)*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl1*(1.0+kappa)*kp_gsign1); } 
	delay(gt1/2.0);
	rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}
	delay(gstab);

// second 90, v3 
   	 rgpulse(pw, v3, rof1, 0.0);		

// Lock refocussing pulse
        if (alt_grd[0] == 'y')
        {
	ifzero(v7); 
		{
		rgradient(gradaxis[0],gzlvl1*2.0*kappa*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl1*2.0*kappa*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }  
		}
         elsenz(v7); 
		{
		rgradient(gradaxis[0],-1.0*gzlvl1*2.0*kappa*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl1*2.0*kappa*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
         endif(v7);
        }
        else 
	{
	rgradient(gradaxis[0],gzlvl1*2.0*kappa*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl1*2.0*kappa*kp_gsign1); } 
	delay(gt1/2.0); 
	rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}
	delay(gstab);

// Spoiler gradient balancing pulse 
        if (alt_grd[0] == 'y')
        {
	ifzero(v7); 
		{
		rgradient(gradaxis[0],gzlvl3*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl3*kp_gsign1); } 
		delay(gt3); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); } 
		}
         elsenz(v7); 
		{
		rgradient(gradaxis[0],-1.0*gzlvl3*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl3*kp_gsign1); } 
		delay(gt3); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
         endif(v7);
        }
        else 
	{
	rgradient(gradaxis[0],gzlvl3*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl3*kp_gsign1); } 
	delay(gt3); 
	rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}
	delay(gstab);

// diffusion delay 
         delcor = del-4.0*pw-3.0*rof1-2.0*gt1-5.0*gstab-gt3;
         if (satmode[1] == 'y')
           {
             obspower(satpwr);
             if (satfrq != tof)
             obsoffset(satfrq);
             rgpulse(delcor,zero,rof1,rof1);
             if (satfrq != tof)
             obsoffset(tof);
             obspower(tpwr);
           }
	 else delay(del-4.0*pw-3.0*rof1-2.0*gt1-5.0*gstab-gt3); 

//Lock refocussing pulse
        if (alt_grd[0] == 'y')
        {
	ifzero(v7); 
		{
		rgradient(gradaxis[0],2.0*kappa*gzlvl1*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],2.0*kappa*gzlvl1*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }  
		}
         elsenz(v7); 
		{
		rgradient(gradaxis[0],-2.0*kappa*gzlvl1*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-2.0*kappa*gzlvl1*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
         endif(v7);
        }
        else 
	{
	rgradient(gradaxis[0],2.0*kappa*gzlvl1*kp_gsign0);
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],2.0*kappa*gzlvl1*kp_gsign1); } 
	delay(gt1/2.0); 
	rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}
	delay(gstab);

// third 90, v4 
   	 rgpulse(pw, v4, rof1, 0.0);	

//3rd main gradient pulse
        if (alt_grd[0] == 'y')
        {
	ifzero(v7); 
		{
		rgradient(gradaxis[0],gzlvl1*(1.0-kappa)*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl1*(1.0-kappa)*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); } 
		}
         elsenz(v7); 
		{
		rgradient(gradaxis[0],-1.0*gzlvl1*(1.0-kappa)*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*gzlvl1*(1.0-kappa)*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
         endif(v7);
        }
        else 
	{
	rgradient(gradaxis[0],gzlvl1*(1.0-kappa)*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],gzlvl1*(1.0-kappa)*kp_gsign1); } 
	delay(gt1/2.0);
	rgradient(gradaxis[0],0.0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}
	delay(gstab);

// second 180, v5 
	 rgpulse(pw*2.0, v5, rof1, rof1);	

//4th main grad. pulse
        if (alt_grd[0] == 'y')
        {
	ifzero(v7); 
		{
		rgradient(gradaxis[0],-1.0*(1.0+kappa)*gzlvl1*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*(1.0+kappa)*gzlvl1*kp_gsign1); } 
		delay(gt1/2.0); 
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); } 
		}
         elsenz(v7); 
		{
		rgradient(gradaxis[0],1.0*(1.0+kappa)*gzlvl1*kp_gsign0); 
		if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],1.0*(1.0+kappa)*gzlvl1*kp_gsign1); }
		delay(gt1/2.0);
		rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
		}
         endif(v7);
        }
        else 
	{
	rgradient(gradaxis[0],-1.0*(1.0+kappa)*gzlvl1*kp_gsign0); 
	if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],-1.0*(1.0+kappa)*gzlvl1*kp_gsign1); } 
	delay(gt1/2.0); 
	rgradient(gradaxis[0],0.0); if ((gradaxis[1]=='x') || (gradaxis[1]=='y') || (gradaxis[1]=='z')) { rgradient(gradaxis[1],0.0); }
	}

// small flip angle, orthogonal to the last 90 
	if (oneshot45_flg[0] == 'y')
	{
   	 delay(gstab);
	 rgpulse(kp_beta/90.0*pw,v6,rof1,0.0);	
	}
	else
   	 delay(gstab+2.0*pw/PI);
      
   if (lkgate_flg[0] == 'y')  lk_sample(); // turn on lock sampling unless method 'k' is used 

//Acquisition
      
//filter delay (Hoult) for inova; adjust is needed
//delay(1.0/(getval("fb")*1.3))

  setacqmode(WACQ|NZ);	//use this line only for vnmrs console; comment this out in inova

	obsblank();	
	delay(rof2);
	startacq(alfa);
  status(C);	
	acquire(np,1.0/sw);
	rcvroff();	
   if (lkgate_flg[0] == 'k')  lk_sample(); // turn on lock sampling unless method 'k' is used 

   }
   else
	{
      rgpulse(pw,oph,rof1,rof2);
   status(C);
	}



}

/****************************************************************************
256 (or 16,32,64,128) steps phase cycle
Order of cycling: v1 (0,2), v4(0,2), v1(1,3), v4(1,3), v5(0,2), v3(0,2),
v5(1,3) v2(0,2)

receiver = v1-2*v2+v3-v4+2*v5

Coherence pathway :

	90	180	 90		90	180	  Acq.
+2

+1                ________                _______
                 |        |              |       |
 0------|        |        |--------------|       |
        |        |                               |
-1      |________|                               |____________________

-2
	v1       v2        v3             v4      v5
****************************************************************************/

